<?php

use Tygh\Settings;
use Tygh\Registry;
use Tygh\Http;

function fn_update_dreamkas_settings($settings)
{
    if (isset($settings['payment_ids'])) {
        $settings['payment_ids'] = implode(',',$settings['payment_ids']);
    }
    
    foreach ($settings as $setting_name => $setting_value) {
        Settings::instance()->updateValue($setting_name, $setting_value);
    }
}

function fn_get_dreamkas_settings($lang_code = DESCR_SL)
{
    $dk_settings = Settings::instance()->getValues('dreamkas', 'ADDON');
    
    if (!empty($dk_settings['general']['payment_ids'])) {
        $dk_settings['general']['payment_ids'] = explode(',',$dk_settings['general']['payment_ids']);
    }
    
    return $dk_settings['general'];
}

function fn_settings_variants_addons_dreamkas_payments_ids()
{
        $params['status'] = 'A';
        $payments = fn_get_payments($params);
        if(!empty($payments)) {
            foreach ($payments as $payment_id => $payment) {
                $_payments[$payment_id] = $payment['payment'];
            }
        }

        return $_payments;
}

function fn_dk_post_cheque($order_info)
{
    
    $settings = Registry::get('addons.dreamkas');
    $url = 'receipts';
    
    $positions = fn_dk_prepare_items($order_info, $settings);
    $phone = empty($order_info['phone'])?$order_info['b_phone']:$order_info['phone'];
    
    $data = array(
        "deviceId" => $settings['device_id'],
        "type" => "SALE",
        "timeout" => 180,
        "taxMode" => $settings['tax_mode'],
        "positions" => $positions,
        "payments" => array(
            array(
                "sum" => $order_info['total']*100,
                "type" => "CASHLESS"
            )
        ),
        "attributes" => array(
          "email" => $order_info['email'],
          "phone" => $order_info['phone']//"+71239994499"
        ),
        "total" => array(
          "priceSum" => $order_info['total']*100
        )
    );

    $result = fn_dk_request($url, $data);

    return $result;
}

function fn_dk_prepare_items($order_info, $settings) {
    
    $items = array();
    
    foreach ($order_info['products'] as $product) {
        $items[] = array(
            "name"=> $product['product'],
            "type"=> "COUNTABLE",
            "quantity"=> $product['amount'],
            "price"=> $product['price']*100,
            "priceSum"=> $product['subtotal']*100,
            "tax"=> "$settings[tax_type]",
            "taxSum"=> $product['tax_value']*100
        );
    }
    
    if($order_info['shipping_cost']>0) {
        $shipping = reset($order_info['shipping']);
        $taxes = reset($shipping['taxes']);
        $items[] = array(
            "name"=> __('shipping').': '.$shipping['shipping'],
            "type"=> "COUNTABLE",
            "quantity"=> 1,
            "price"=> $order_info['shipping_cost']*100,
            "priceSum"=> $order_info['shipping_cost']*100,
            "tax"=> "$settings[tax_type]",
            "taxSum"=> $taxes['tax_subtotal']*100
        );
    }
    
    return $items;
}

function fn_dk_get_operation_status($dk_id) {

    $data = array();
    $url = 'operations/'.$dk_id;
    $result = fn_dk_request($url, $data, 'get');
    
    return $result;
}

function fn_dk_request($url='', $data=array(), $method='post') {
    
    $api_url = 'https://kabinet.dreamkas.ru/api/';
    $access_token =  Registry::get('addons.dreamkas.access_token');

    if($method=='get') {
        $response = Http::get($api_url.$url, $data, array(
                'headers' => array(
                    "Authorization: Bearer ".$access_token
                )
            ));
    } else {
        $response = Http::post($api_url.$url, json_encode($data), array(
                'headers' => array(
                    "Content-Type: application/json",
                    "Authorization: Bearer ".$access_token
                )
            ));
    }
    
    if (!empty($response)) {
        $data = json_decode($response, true);

        if(Registry::get('runtime.mode')=='update_dreamkas' && (substr($data['status'], 0, 1)==4)) {
            fn_set_notification('E', __('error'), $data['message']);
            return false;
        }
        if(Registry::get('runtime.mode')=='update_dreamkas' && $data['status']=='ERROR') {
            fn_set_notification('E', __('error'), $data['data']['error']['message']);
            //return false;
        }
        return $data;
     }
        
   return false;
}

function fn_dreamkas_get_order_info(&$order, $additional_data) {

}

function fn_dreamkas_get_orders(&$params, &$fields, &$sortings, &$condition, &$join)
{
    if (!empty($params['dk_status'])) {
        $condition .= db_quote(' AND ?:orders.dk_status IN (?a)', $params['dk_status']);
    }
    return true;
}

function fn_dreamkas_change_order_status(&$status_to, &$status_from, &$order_info, &$force_notification, &$order_statuses, &$place_order)
{

    $settings = Registry::get('addons.dreamkas');
    $order_id = $order_info['order_id'];
    $payment_id = $order_info['payment_id'];
    $payment_ids = array_keys($settings['payments_ids']);
            
    if ($status_to=='P' && !empty($payment_ids) && in_array($payment_id, $payment_ids)) {
        $dk_id = db_get_field("SELECT dk_id FROM ?:orders WHERE order_id = ?i", $_REQUEST['order_id']);
        if(empty($dk_id)) {
            $cheque_info = fn_dk_post_cheque($order_info);
            if(!empty($cheque_info['id'])){
                $data = array(
                    'dk_id' => $cheque_info['id'],
                    'dk_timestamp' => strtotime($cheque_info['createdAt']),
                    'dk_status' => $cheque_info['status'],
                );
                db_query("UPDATE ?:orders SET ?u WHERE order_id = ?i", $data, $order_id);
            }
        }
    }
}

function fn_dreamkas_pre_get_orders($params, &$fields, $sortings, $get_totals, $lang_code) {
    $fields[] = '?:orders.dk_id';
    $fields[] = '?:orders.dk_timestamp';
    $fields[] = '?:orders.dk_status';
}